import * as React from 'react';
import TextField, { BaseTextFieldProps } from '@material-ui/core/TextField';
import { MenuItem, Zoom } from '@material-ui/core';
import { createStyles, makeStyles } from '@material-ui/core/styles';
import { LightTooltip } from './LightTooltip';

export interface ISelectOption {
  label: string;
  value: string;
  tooltip?: any;
  invalid?: boolean;
}

const useStyles = makeStyles(() =>
  createStyles({
    label: {
      color: 'var(--jp-input-border-color)',
      fontSize: 'var(--jp-ui-font-size2)',
    },
    input: {
      color: 'var(--jp-ui-font-color1)',
    },
    textField: {
      width: '100%',
    },
    menu: {
      backgroundColor: 'var(--jp-layout-color1)',
      color: 'var(--jp-ui-font-color1)',
    },
    helperLabel: {
      color: 'var(--jp-info-color0)',
    },
  }),
);

interface SelectProps extends BaseTextFieldProps {
  index: number;
  values: ISelectOption[];
  variant?: 'filled' | 'standard' | 'outlined';
  updateValue: Function;
}

export const Select: React.FunctionComponent<SelectProps> = props => {
  const classes = useStyles({});

  const {
    index,
    value,
    values,
    helperText = null,
    variant = 'outlined',
    updateValue,
    ...rest
  } = props;

  const disableMenuItem = (event: React.MouseEvent, invalidOption: boolean) => {
    if (invalidOption) {
      event.stopPropagation();
    }
  };

  const getOptionClassNames = (option: any) => {
    const classNames: string[] = [];
    if (option.tooltip) {
      classNames.push('menu-item-tooltip');
    }
    return classNames.join(' ');
  };

  return (
    // @ts-ignore
    <TextField
      select
      {...rest}
      margin="dense"
      value={value}
      variant={variant}
      className={classes.textField}
      onChange={evt =>
        updateValue((evt.target as HTMLInputElement).value, index)
      }
      InputLabelProps={{
        classes: { root: classes.label },
        shrink: value !== '',
      }}
      InputProps={{ classes: { root: classes.input } }}
      SelectProps={{ MenuProps: { PaperProps: { className: classes.menu } } }}
      FormHelperTextProps={{ classes: { root: classes.helperLabel } }}
    >
      {values.map((option: any) => (
        <MenuItem
          key={option.value}
          value={option.value}
          disabled={!!option.invalid}
          className={getOptionClassNames(option)}
        >
          {option.tooltip ? (
            <LightTooltip
              title={option.tooltip}
              placement="top-start"
              interactive={!(typeof option.tooltip === 'string')}
              TransitionComponent={Zoom}
            >
              <div
                className="menu-item-label"
                onClick={ev => disableMenuItem(ev, !!option.invalid)}
              >
                {option.label}
              </div>
            </LightTooltip>
          ) : (
            option.label
          )}
        </MenuItem>
      ))}
    </TextField>
  );
};
